package com.sun.jndi.ldap;

import java.util.EventObject;
import javax.naming.ldap.UnsolicitedNotificationEvent;
import javax.naming.event.NamingExceptionEvent;
import javax.naming.ldap.UnsolicitedNotification;
import java.util.Iterator;
import javax.naming.directory.SearchControls;
import javax.naming.NamingException;
import javax.naming.event.NamespaceChangeListener;
import javax.naming.event.ObjectChangeListener;
import javax.naming.event.NamingListener;
import javax.naming.ldap.UnsolicitedNotificationListener;
import java.util.Vector;
import java.util.Hashtable;

final class EventSupport
{
    private static final boolean debug = false;
    private LdapCtx ctx;
    private Hashtable<NotifierArgs, NamingEventNotifier> notifiers;
    private Vector<UnsolicitedNotificationListener> unsolicited;
    private EventQueue eventQueue;
    
    EventSupport(LdapCtx ctx) {
        this.notifiers = new Hashtable(11);
        this.unsolicited = null;
        this.ctx = ctx;
    }
    
    synchronized void addNamingListener(String s, int n, NamingListener namingListener) throws NamingException {
        if (namingListener instanceof ObjectChangeListener || namingListener instanceof NamespaceChangeListener) {
            NotifierArgs notifierArgs = new NotifierArgs(s, n, namingListener);
            NamingEventNotifier namingEventNotifier = (NamingEventNotifier)this.notifiers.get(notifierArgs);
            if (namingEventNotifier == null) {
                this.notifiers.put(notifierArgs, new NamingEventNotifier(this, this.ctx, notifierArgs, namingListener));
            }
            else {
                namingEventNotifier.addNamingListener(namingListener);
            }
        }
        if ((namingListener instanceof UnsolicitedNotificationListener)) {
            if (this.unsolicited == null) {
                this.unsolicited = new Vector(3);
            }
            this.unsolicited.addElement((UnsolicitedNotificationListener)namingListener);
        }
    }
    
    synchronized void addNamingListener(String s, String s2, SearchControls searchControls, NamingListener namingListener) throws NamingException {
        if (namingListener instanceof ObjectChangeListener || namingListener instanceof NamespaceChangeListener) {
            NotifierArgs notifierArgs = new NotifierArgs(s, s2, searchControls, namingListener);
            NamingEventNotifier namingEventNotifier = (NamingEventNotifier)this.notifiers.get(notifierArgs);
            if (namingEventNotifier == null) {
                this.notifiers.put(notifierArgs, new NamingEventNotifier(this, this.ctx, notifierArgs, namingListener));
            }
            else {
                namingEventNotifier.addNamingListener(namingListener);
            }
        }
        if ((namingListener instanceof UnsolicitedNotificationListener)) {
            if (this.unsolicited == null) {
                this.unsolicited = new Vector(3);
            }
            this.unsolicited.addElement((UnsolicitedNotificationListener)namingListener);
        }
    }
    
    synchronized void removeNamingListener(NamingListener obj) {
        Iterator<NamingEventNotifier> iterator = this.notifiers.values().iterator();
        while (iterator.hasNext()) {
            NamingEventNotifier namingEventNotifier = (NamingEventNotifier)iterator.next();
            if (namingEventNotifier != null) {
                namingEventNotifier.removeNamingListener(obj);
                if (!namingEventNotifier.hasNamingListeners()) {
                    namingEventNotifier.stop();
                    iterator.remove();
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        if (this.unsolicited != null) {
            this.unsolicited.removeElement(obj);
        }
    }
    
    synchronized boolean hasUnsolicited() {
        return this.unsolicited != null && this.unsolicited.size() > 0;
    }
    
    synchronized void removeDeadNotifier(NotifierArgs key) {
        this.notifiers.remove(key);
    }
    
    synchronized void fireUnsolicited(Object o) {
        if (this.unsolicited == null || this.unsolicited.size() == 0) {
            return;
        }
        if ((o instanceof UnsolicitedNotification)) {
            this.queueEvent(new UnsolicitedNotificationEvent(this.ctx, (UnsolicitedNotification)o), this.unsolicited);
        }
        else if ((o instanceof NamingException)) {
            this.queueEvent(new NamingExceptionEvent(this.ctx, (NamingException)o), this.unsolicited);
            this.unsolicited = null;
        }
        return;
    }
    
    synchronized void cleanup() {
        if (this.notifiers != null) {
            Iterator<NamingEventNotifier> iterator = this.notifiers.values().iterator();
            while (iterator.hasNext()) {
                ((NamingEventNotifier)iterator.next()).stop();
            }
            this.notifiers = null;
        }
        if (this.eventQueue != null) {
            this.eventQueue.stop();
            this.eventQueue = null;
        }
    }
    
    synchronized void queueEvent(EventObject eventObject, Vector<? extends NamingListener> vector) {
        if (this.eventQueue == null) {
            this.eventQueue = new EventQueue();
        }
        this.eventQueue.enqueue(eventObject, (Vector)vector.clone());
    }
}
